/*
 * tkimg.h --
 *	Declarations for internal routines used by Img package.
 */

#ifndef __TKIMG_H__
#define __TKIMG_H__

#include "tk.h"

#define TKIMG_VERSION "1.3.99"

#if BUILD_tkimg
#	define TKIMGAPI DLLEXPORT
#elif USE_TKIMG_STUBS
#	define TKIMGAPI /*nothing*/
#else
#	define TKIMGAPI DLLIMPORT
#endif

/*
 * The tkimg_MFile routines encapsulate the different input sources
 * and output destinations for photo image formats (Tcl_Channels,
 * binary data in Tcl_Obj's, base-64-encoded data, etc.)
 */
typedef struct tkimg_MFile {
    Tcl_DString *buffer;/* pointer to dynamical string */
    char *data;		/* mmencoded source string */
    int c;		/* bits left over from previous char */
    int state;		/* decoder state (0-4 or IMG_DONE) */
    int length;		/* length of physical line already written */
} tkimg_MFile;

#define IMG_SPECIAL	(1<<8)
#define IMG_PAD		(IMG_SPECIAL+1)
#define IMG_SPACE	(IMG_SPECIAL+2)
#define IMG_BAD		(IMG_SPECIAL+3)
#define IMG_DONE	(IMG_SPECIAL+4)
#define IMG_CHAN        (IMG_SPECIAL+5)
#define IMG_STRING	(IMG_SPECIAL+6)

/*
 * Function prototypes:
 */

TKIMGAPI Tcl_Channel tkimg_OpenFileChannel(
	Tcl_Interp *, const char *fileName, int permissions);
TKIMGAPI int tkimg_ReadInit(Tcl_Obj *data, int c, tkimg_MFile *handle);
TKIMGAPI void tkimg_WriteInit(Tcl_DString *buffer, tkimg_MFile *handle);
TKIMGAPI int tkimg_Getc(tkimg_MFile *handle);
TKIMGAPI int tkimg_Read(tkimg_MFile *handle, void * dst, int count);
TKIMGAPI void tkimg_Putc(int c, tkimg_MFile * handle);
TKIMGAPI void tkimg_Finish(tkimg_MFile *handle);
TKIMGAPI int tkimg_Write(tkimg_MFile *handle, const char * src, int count);

/*
 * Convenience macros dealing with the ever-varying signatures 
 * [TIP 98, TIP 116] of various Tk_Photo* routines.
 *
 * Default to TK_PHOTO_COMPOSITE_SET (not TK_PHOTO_COMPOSITE_OVERLAY)
 * to have the initial load of images with alpha channels be correct.
 */

#if ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION > 4))
/* TIP 116 */
#define tkimg_PhotoPutBlockTk(interp,hdl,b,x,y,w,h) \
    	Tk_PhotoPutBlock(interp, hdl, b, x, y, w, h, TK_PHOTO_COMPOSITE_SET)
#elif ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION > 3))
/* TIP 98 */
#define tkimg_PhotoPutBlockTk(interp,hdl,b,x,y,w,h) \
	Tk_PhotoPutBlock(hdl, b, x, y, w, h, TK_PHOTO_COMPOSITE_SET)
#else
#define tkimg_PhotoPutBlockTk(interp,hdl,b,x,y,w,h) \
	Tk_PhotoPutBlock(hdl, b, x, y, w, h)
#endif

#if ((TK_MAJOR_VERSION == 8) && (TK_MINOR_VERSION > 4))
/* TIP 116 */
#define tkimg_PhotoExpand(interp,hdl,w,h)  Tk_PhotoExpand(interp, hdl, w, h)
#else
#define tkimg_PhotoExpand(interp,hdl,w,h)  Tk_PhotoExpand(hdl, w, h)
#endif

#ifndef CONST84
/* TIP 27 */
#   define CONST84 /*empty*/
#endif

#endif /* __TKIMG_H__ */
